import bpy
from . import utils
    
from bpy.props import (BoolProperty,
                       EnumProperty,
                       StringProperty,
                       IntProperty,
                       FloatProperty,
                       PointerProperty,
                       CollectionProperty
                       )
                       
def update_web_search(self, context):
    utils.__y__()
    
def change_font(self, context):
    utils.__1__()
    
class FM_FontItem(bpy.types.PropertyGroup):
    name: StringProperty(name="字体名称")
    path: StringProperty(name="字体路径")
    
class FM_Props(bpy.types.PropertyGroup):
  
    tab: EnumProperty(
        name = "选项卡",
        description = "Switch between Tabs",
        items = [
            ("1", "收藏", "Display Custom Fonts", 'SOLO_ON', 1),
            ("2" , "系统" , "Display System fonts", 'HOME', 2),
            ("3", "下载", "Display Downloaded fonts", 'IMPORT', 3),
            ("4", "在线", "Display Web Fonts", 'URL', 4),
            ("5", "自定义", "Display Custom Fonts", 'USER', 5),
        ],
        default = "2"
        )
        
    views: EnumProperty(
        name = "Views",
        description = "Switch between Views",
        items = [
            ("1" , "Previews" , "Display fonts in Previews"),
            ("2", "List", "Display fonts in List"),
        ],
        default = "1"
        )
        
    search: StringProperty(name="Search", default="", options={'TEXTEDIT_UPDATE','SKIP_SAVE'})
    web_search: StringProperty(name="Search", default="", update=update_web_search)
    go_to_page: IntProperty(
        name="Jump To Page",
        description = "Jump to specific page",
        default = 1,
        min=1,
        step=1
    )
    
    thumb_size: IntProperty(
        name="Thumb Size",
        description = "The size of the Thumbnails",
        default = 1,
        min=1,
        max=6,
        step=1
    )
    
    reload: BoolProperty(
        name="Reload Fonts",
        description = "Reload Fonts",
        default = True
    )
    
    font_folder : StringProperty(
        name="字体文件夹",
        description="Path to the font folder/library",
        default="",
        subtype="DIR_PATH"
    )
        
    active_page: StringProperty(name="Active page", default="/top.php?page=1")
    cached_page: StringProperty(name="Cached page", default="")
    next_page: StringProperty(name="下一页", default="")
    prev_page: StringProperty(name="上一页", default="")
    
    web_search_filters: BoolProperty(
        name="显示Web搜索筛选器",
        description = "显示Web搜索筛选器",
        default = False
    )
    
    web_categories: BoolProperty(
        name="Browse Categories",
        description = "Browse Categories",
        default = False
    )
    
    hot_reload: BoolProperty(
        name="Hot Reload",
        description = "Hot Reload",
        default = False
    )
    
    sortBy: EnumProperty(
        name = "Sort By",
        description = "Select how you want to sort the fonts",
        items = [
            ("1" , "名称" , "Sort by Name"),
            ("2", "热门使用", "Sort by Popularity"),
            ("3", "最新的", "Sort by Newest"),
        ],
        default = "2"
        )
        
    numFonts: EnumProperty(
        name = "Number of Fonts",
        description = "Number of Fonts per Page",
        items = [
            ("1" , "25" , "Show 25 Fonts"),
            ("2", "50", "Show 50 Fonts"),
            ("3", "100", "Show 100 Fonts"),
            ("3", "200", "Show 200 Fonts"),
        ],
        default = "1"
        )
        
    show_pD: BoolProperty(name="Public domain / GPL / OFL", default=False)
    show_fR: BoolProperty(name="100% Free", default=False)
    show_pU: BoolProperty(name="Free for Personal Use", default=False)
    show_dW: BoolProperty(name="Donationware", default=False)
    show_sW: BoolProperty(name="Shareware", default=False)
    show_dM: BoolProperty(name="Demo", default=False)
    show_uK: BoolProperty(name="Unknown", default=False)
    
        
classes = (FM_Props,FM_FontItem,)


def register():
    for cls in classes:
        bpy.utils.register_class(cls)
        
    bpy.types.WindowManager.fm_props = PointerProperty(type=FM_Props)
    bpy.types.WindowManager.fm_fonts_collection = CollectionProperty(type=FM_FontItem)
    bpy.types.WindowManager.fm_fonts_collection_index = bpy.props.IntProperty(name="Index for fm_fonts_collection", default=0, update=change_font)
    from pathlib import Path
    print(f"{Path(__file__).parent.name} 插件由xz-blender购买分享!感谢支持!")


def unregister():
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)
        
    del bpy.types.WindowManager.fm_props
    del bpy.types.WindowManager.fm_fonts_collection
    del bpy.types.WindowManager.fm_fonts_collection_index