import bpy
import os
from .. import utils
import threading

if not hasattr(bpy.types.WindowManager, "downloading_fonts"):
    bpy.types.WindowManager.downloading_fonts = bpy.props.StringProperty(default="")

class FM_OT_apply_font(bpy.types.Operator):
    bl_idname = 'fm.apply_font'
    bl_label = 'Apply this Font'
    bl_description = 'Apply this Font'
    bl_options = {'UNDO'}
    
    Font: bpy.props.StringProperty()

    def execute(self, context: bpy.types.Context):
        utils.__0__(self.Font, self)
        return {'FINISHED'}
        
class FM_OT_apply_filters(bpy.types.Operator):
    bl_idname = 'fm.apply_filters'
    bl_label = 'Apply Filters'
    bl_description = 'Apply Filters'
    bl_options = {'UNDO'}

    def execute(self, context: bpy.types.Context):
        fm_props = utils.__e__()
        fm_props.hot_reload = True
        fm_props.web_search_filters = False
        return {'FINISHED'}
        
class FM_OT_category(bpy.types.Operator):
    bl_idname = 'fm.category'
    bl_label = 'Select Category'
    bl_description = 'Select this Category'
    bl_options = {'UNDO'}
    
    cat: bpy.props.IntProperty()

    def execute(self, context: bpy.types.Context):
        fm_props = utils.__e__()
        fm_props.web_categories = False
        fm_props.web_search = ""
        u = '/theme.php?cat='+str(self.cat)+'&'
        if self.cat == 900:
            u = "/bitmap.php?"
        elif self.cat == 99:
            u = "/top.php?"
        fm_props.active_page = u+'page=1'
        return {'FINISHED'}
        
class FM_OT_jump_to_page(bpy.types.Operator):
    bl_idname = 'fm.jump_to_page'
    bl_label = 'Jump to Page'
    bl_description = 'Jump to specific Page'
    bl_options = {'UNDO'}
    
    pMax: bpy.props.IntProperty()
         
    def execute(self, context: bpy.types.Context):
        utils.__2__(self)        
        return {'FINISHED'}
        
class FM_OT_download_font(bpy.types.Operator):
    bl_idname = 'fm.download_font'
    bl_label = 'Download Font'
    bl_description = 'Download this Font'
    bl_options = {'UNDO'}

    font: bpy.props.StringProperty()

    def execute(self, context: bpy.types.Context):
        wm = context.window_manager
        downloading_fonts = wm.downloading_fonts.split(',')

        if self.font in downloading_fonts:
            self.report({'WARNING'}, "Font is already downloading")
            return {'CANCELLED'}

        fm_props = utils.__e__()
        fonts_data = utils.__h__()

        for font in fonts_data['fonts']:
            if font['thumbnail_url'].split('/')[-1].split('.')[0] == self.font:
                url = 'https:' + font['download_link']
                font_name = self.font
                
                downloading_fonts.append(self.font)
                wm.downloading_fonts = ','.join(downloading_fonts)

                def on_download_complete(success, path=None, error=None):
                    downloading_fonts = wm.downloading_fonts.split(',')
                    if self.font in downloading_fonts:
                        downloading_fonts.remove(self.font)
                        wm.downloading_fonts = ','.join(downloading_fonts)

                    if success:
                        self.report({'INFO'}, f"Download completed: {path}")
                        utils.__x__(self, context)
                    else:
                        self.report({'ERROR'}, f"Download failed: {error}")

                threading.Thread(target=utils.__r__, args=(url, on_download_complete, font_name)).start()
                break
        return {'FINISHED'}
        
class FM_OT_font_info(bpy.types.Operator):
    bl_idname = "fm.font_info"
    bl_label = "Font Info"
    bl_description = "Open Font Creator's URL"
    bl_options = {'UNDO'}
    
    font: bpy.props.StringProperty()
            
    def execute(self,context):        
        fm_props = utils.__e__()
        fonts_data = utils.__h__()
        
        for font in fonts_data['fonts']:
            if font['thumbnail_url'].split('/')[-1].split('.')[0] == self.font:
                if font['creator_url'] == 'No creator URL':
                    self.report({'INFO'}, "Webpage not available!")
                else:
                    bpy.ops.wm.url_open(url=font['creator_url'])
        return {'FINISHED'}
 
class FM_OT_favourite(bpy.types.Operator):
    bl_idname = "fm.favourite"
    bl_label = "Favourite"
    bl_description = "Add or Remove from Favourite fonts"
    bl_options = {'UNDO'}
    
    font: bpy.props.StringProperty()
            
    def execute(self,context):        
        fm_props = utils.__e__()
        if utils.__4__(self.font):
            font_paths = [self.font]
        else:
            font_paths = utils.__c__(self.font)
        fonts_list, favourites_list = utils.__s__()

        for path in font_paths:
            if path in favourites_list:
                favourites_list.remove(path)
                self.report({'INFO'}, "Font has been removed from favourites")
            else:
                favourites_list.add(path)
                self.report({'INFO'}, "Font has been added to favourites")

        utils.__t__(fonts_list, favourites_list)
        
        return {'FINISHED'}
        
class FM_OT_delete(bpy.types.Operator):
    bl_idname = "fm.delete"
    bl_label = "Delete Font"
    bl_description = "Delete this Font"
    bl_options = {'UNDO'}
    
    font: bpy.props.StringProperty()
            
    def execute(self,context):        
        fm_props = utils.__e__()
        
        font_paths = utils.__c__(self.font)
        fonts_list, favourites_list = utils.__s__()
        
        for path in font_paths:
            if utils.__6__(path):
                if path in favourites_list:
                    favourites_list.remove(path)
                if self.font in fonts_list:
                    fonts_list.remove(self.font)
                
                self.report({'INFO'}, "The Font has been deleted")

        utils.__t__(fonts_list, favourites_list)
        
        return {'FINISHED'}
        
class FM_OT_font_use(bpy.types.Operator):
    bl_idname = "fm.font_use"
    bl_label = "Apply Font"
    bl_description = "Apply this Font"
    bl_options = {'UNDO'}
    
    font: bpy.props.StringProperty()
            
    def execute(self,context):        
        fm_props = utils.__e__()
        
        font_paths = utils.__c__(self.font)
        utils.__0__(font_paths[0], self)
        fm_props.search = ""
        if len(font_paths) > 1:
            fm_props.search = utils.__3__(self.font)
            fm_props.tab = '3'
        
        return {'FINISHED'}
        
class FM_OT_visit_next(bpy.types.Operator):
    bl_idname = 'fm.visit_next'
    bl_label = 'Go to Next Page'
    bl_description = 'Change Page to the next page'
    bl_options = {'UNDO'}
         
    @classmethod
    def poll(cls, context):
        fm_props = utils.__e__()
        if fm_props.next_page:
            return True

    def execute(self, context: bpy.types.Context):
        fm_props = utils.__e__()
        _page = fm_props.next_page
        if _page:
            fm_props.active_page = '/'+_page
        return {'FINISHED'}
        
class FM_OT_visit_prev(bpy.types.Operator):
    bl_idname = 'fm.visit_prev'
    bl_label = 'Go to Previous Page'
    bl_description = 'Change Page to the previous page'
    bl_options = {'UNDO'}
         
    @classmethod
    def poll(cls, context):
        fm_props = utils.__e__()
        if fm_props.prev_page:
            return True

    def execute(self, context: bpy.types.Context):
        fm_props = utils.__e__()
        _page = fm_props.prev_page
        if _page:
            fm_props.active_page = '/'+_page
        return {'FINISHED'}
