import bpy
from .utils import install_pillow, install_beautifulsoup

class InstallDependencies:
    '''Base class for an operator that installs Pillow.

    Usage:
    -   Inherit bpy.types.Operator and InstallPillow.
    -   Make sure to set bl_idname, it must be unique.
    '''
    bl_label = 'Install Dependencies'
    bl_description = '.\n'.join((
        'Install the dependent Python Libraries',
        'This could take a few minutes',
    ))
    bl_options = {'REGISTER', 'INTERNAL'}

    def execute(self: bpy.types.Operator, context: bpy.types.Context) -> set:
        successfulInstallation = True
        self.report({'INFO'}, 'Installing and Updating Dependencies')
        bpy.context.window.cursor_set("WAIT")
        
        if install_pillow():
            pass
        else:
            self.report({'WARNING'}, 'Failed to install Pillow')
            successfulInstallation = False
        if install_beautifulsoup():
            pass
        else:
            self.report({'WARNING'}, 'Failed to install BeautifulSoup')
            successfulInstallation = False
            
        bpy.context.window.cursor_set("DEFAULT")
        
        if successfulInstallation:
            self.report({'INFO'}, 'Successfully installed and updated all Dependencies')
        else:
            self.report({'INFO'}, "Some/All of the dependencies failed to install or update")
            self.report({'ERROR'}, "Please ensure you are connected to the internet!")
            

        return {'FINISHED'}
